//----------------------------------------------------------------------------
// pacman-like game
// copyright 2010 christopher rasmussen
// university of delaware
// v. 1.07
// * added score/time elapsed drawing
// * crossover collisions now detected
// * added wind terrain
// * added blue_ghost_wait_counter
// * starting to move functions, variables over to GameState class; making it const in call to update()
// * added "-blue" option for blue ghosts/no blue ghosts; saving .lay; new quit key
// * moved MazeRunner class stuff to MazeRunner.hh, MazeRunner.cpp
// * reworked handling of command-line flags; added "-n" option to run multiple games without display

//----------------------------------------------------------------------------

#include "MazeRunner.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

int main(int argc, char **argv)
{
  MazeRunner *MR;

  // initialize random number generator

  srand(time(0));

  // initialize

  int maze_rows = DEFAULT_ROWS;
  int maze_cols = DEFAULT_COLS;
  int blue_period = NO_BLUE_GHOST;
  bool do_random = true;
  string filename;
  int num_games = 1;
  bool do_step = false;
  bool self_play = false;
  int max_turns = MAX_ARENA_UPDATES;

  for (int i = 1; i < argc; i++) {
    if (!strcmp(argv[i], "-blue")) 
      blue_period = STANDARD_BLUE_GHOST;
    else if (isdigit(argv[i][0])) {
      maze_rows = atoi(argv[i]);
      maze_cols = atoi(argv[i + 1]);
      i++;
    }
    else if (!strcmp(argv[i], "-n")) {
      num_games = atoi(argv[i + 1]);
      i++;
      if (num_games < 1) {
	cout << "Number of games must be >= 1\n";
	exit(1);
      }
    }
    else if (!strcmp(argv[i], "-step")) {
      do_step = true;
    }
    else if (!strcmp(argv[i], "-auto")) {
      self_play = true;
    }
    else if (!strcmp(argv[i], "-t")) {
      max_turns = atoi(argv[i + 1]);
      i++;
      if (max_turns < 1) {
	cout << "Number of turns must be >= 1\n";
	exit(1);
      }
    }
    else {
      do_random = false;
      filename = string(argv[i]);
    }
  }

  if (do_random)
    MR = new MazeRunner(maze_rows, maze_cols, blue_period, max_turns);
  else
    MR = new MazeRunner(filename, blue_period, max_turns, self_play);

  MR->do_step = do_step;

  // run

  int result, num_results;
  int num_runner_wins, num_even_wins, num_odd_wins, num_ties;

  for (num_results = 0, num_runner_wins = num_even_wins = num_odd_wins = num_ties = 0; num_results < num_games; num_results++) {

    result = MR->main_loop(num_games == 1);  // 1 game only => display; else no display

    // standard pac-man result

    if (result == RUNNER_WIN)
      num_runner_wins++;

    // arena result

    else if (result == EVEN_WIN)
      num_even_wins++;
    else if (result == ODD_WIN)
      num_odd_wins++;
    else if (result == EVEN_ODD_TIE)
      num_ties++;

    MR->reset();
  }

  // print win statistics

  if (num_even_wins || num_odd_wins || num_ties)
    printf("EVEN win pct = %.1f, ODD win pct = %.1f, TIE pct = %.1f (%i games)\n", 
	   100.0 * (float) num_even_wins / (float) num_results, 
	   100.0 * (float) num_odd_wins / (float) num_results,
	   100.0 * (float) num_ties / (float) num_results,
	   num_games);
  else
    printf("Win percentage = %.1f (%i games)\n", 100.0 * (float) num_runner_wins / (float) num_results, num_games);

  // done

  return 1;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
